#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import Eq
from pyramid.httpexceptions import HTTPFound, HTTPInternalServerError, HTTPNotFound
from pyramid.response import Response
from pyramid.view import render_view_to_response, view_config
from zope.interface import Interface
from zope.traversing.interfaces import ITraversable

from onf_website.reference.forest import IForestTable
from onf_website.reference.forest.model import PARENT_SESSION as RDF_SESSION
from onf_website.reference.forest.model.foret import Foret, InformationForet, ProprietaireForet
from onf_website.shared.forest import FOREST_CONTENT_TYPE, FakeForest, IForestManager
from pyams_alchemy.engine import get_user_session
from pyams_catalog.query import CatalogResultSet
from pyams_skin.interfaces import ISkinnable
from pyams_skin.layer import IPyAMSUserLayer
from pyams_skin.skin import apply_skin
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces.url import DISPLAY_CONTEXT
from pyams_utils.list import boolean_iter
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url, canonical_url
from pyams_workflow.interfaces import IWorkflow, IWorkflowVersions


__docformat__ = 'restructuredtext'


@adapter_config(name='frt',
                context=(Interface, IPyAMSUserLayer),
                provides=ITraversable)
class ForestTraverser(ContextRequestAdapter):
    """Forest namespace traverser"""

    def traverse(self, name, furtherpath=None):
        if not name:
            raise HTTPNotFound()
        forest_id = name.upper()
        if '::' in forest_id:
            forest_id, ignored = forest_id.split('::', 1)
            if not forest_id:
                raise HTTPNotFound()
        request = self.request
        catalog = get_utility(ICatalog)
        params = Eq(catalog['content_type'], FOREST_CONTENT_TYPE) & \
            Eq(catalog['forests'], forest_id)
        has_results, results = boolean_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))
        if has_results:
            forest = next(results)
            workflow = IWorkflow(forest, None)
            if workflow is None:
                raise HTTPInternalServerError()
            versions = IWorkflowVersions(forest, None)
            if versions is None:
                raise HTTPInternalServerError()
            forest = None
            versions = versions.get_versions(workflow.visible_states, sort=True)
            if versions:
                forest = versions[-1]
            if forest is not None:
                location = canonical_url(forest, request, query=request.params)
                if location == request.url:
                    request.annotations[DISPLAY_CONTEXT] = request.context
                    request.context = forest
                    response = render_view_to_response(forest, request, '')
                    return response
                raise HTTPFound(location=location)
        # try to fallback with fake forest content
        session = get_user_session(RDF_SESSION)
        ref_foret, prop, info = session.query(Foret,
                                              ProprietaireForet,
                                              InformationForet) \
            .join(ProprietaireForet, Foret.id_nat_frt == ProprietaireForet.id_nat_frt) \
            .outerjoin(InformationForet, Foret.id_nat_frt == InformationForet.id_nat_frt) \
            .filter(Foret.id_nat_frt == forest_id) \
            .order_by(ProprietaireForet.pourcentage_part, ProprietaireForet.categorie) \
            .first() or (None, None, None)
        if not ref_foret:
            raise HTTPNotFound()
        if info and not info.visible:
            raise HTTPNotFound()
        table = query_utility(IForestTable)
        if (table is None) or (not table.forest_displays.get(prop.categorie)):
            raise HTTPNotFound()
        fake_forest = FakeForest()
        fake_forest.forest_ids = [forest_id]
        return fake_forest


@view_config(route_name='forest_access')
def get_forest_access(request):
    """Direct access to given forest"""
    manager = query_utility(IForestManager)
    if manager is None:
        raise HTTPNotFound()
    forest_id = request.matchdict.get('id_nat_frt')
    if not forest_id:
        raise HTTPNotFound()
    if '::' in forest_id:
        forest_id, ignored = forest_id.split('::', 1)
        if not forest_id:
            raise HTTPNotFound()
    forest_id = forest_id.upper()
    # look for content matching given forest ID
    view_name = ''.join(request.matchdict.get('view'))
    catalog = get_utility(ICatalog)
    params = Eq(catalog['content_type'], FOREST_CONTENT_TYPE) & \
        Eq(catalog['forests'], forest_id)
    has_results, results = boolean_iter(CatalogResultSet(CatalogQuery(catalog).query(params)))
    if has_results:
        forest = next(results)
        workflow = IWorkflow(forest, None)
        if workflow is None:
            raise HTTPInternalServerError()
        versions = IWorkflowVersions(forest, None)
        if versions is None:
            raise HTTPInternalServerError()
        forest = None
        if view_name == 'preview.html':
            forest = versions[str(versions.last_version_id)]
        else:
            versions = versions.get_versions(workflow.visible_states, sort=True)
            if versions:
                forest = versions[-1]
        if forest is not None:
            if view_name:
                location = absolute_url(forest, request, view_name)
            else:
                location = canonical_url(forest, request, query=request.params)
            if location == request.url:
                request.annotations[DISPLAY_CONTEXT] = request.context
                request.context = forest
                response = render_view_to_response(forest, request, view_name)
            else:
                response = Response()
                response.status_code = HTTPFound.code
                response.location = location
            return response
    # try to fallback with fake forest content
    session = get_user_session(RDF_SESSION)
    ref_foret, prop, info = session.query(Foret,
                                          ProprietaireForet,
                                          InformationForet) \
        .join(ProprietaireForet, Foret.id_nat_frt == ProprietaireForet.id_nat_frt) \
        .outerjoin(InformationForet, Foret.id_nat_frt == InformationForet.id_nat_frt) \
        .filter(Foret.id_nat_frt == forest_id) \
        .order_by(ProprietaireForet.pourcentage_part, ProprietaireForet.categorie) \
        .first() or (None, None, None)
    if not ref_foret:
        raise HTTPNotFound()
    if info and not info.visible:
        raise HTTPNotFound()
    table = query_utility(IForestTable)
    if (table is None) or (not table.forest_displays.get(prop.categorie)):
        raise HTTPNotFound()
    fake_forest = FakeForest()
    fake_forest.forest_ids = [forest_id]
    target = fake_forest.__parent__
    site = get_parent(target, ISkinnable)
    apply_skin(request, ISkinnable(site).get_skin(request))
    request.annotations[DISPLAY_CONTEXT] = target
    request.context = fake_forest
    return render_view_to_response(fake_forest, request, view_name)
